@@ -1,5 +1,5 @@
-// Copyright 2008-present Contributors to the OpenImageIO project.
-// SPDX-License-Identifier: BSD-3-Clause
+// Copyright Contributors to the OpenImageIO project.
+// SPDX-License-Identifier: BSD-3-Clause and Apache-2.0
 // https://github.com/OpenImageIO/oiio
 
 
@@ -70,15 +70,19 @@ private:
 }  // namespace
 
 
+
 OIIO_PLUGIN_EXPORTS_BEGIN
 
 OIIO_EXPORT ImageOutput*
 heif_output_imageio_create()
 {
+    extern void oiio_heif_init();
+    oiio_heif_init();
     return new HeifOutput;
 }
 
-OIIO_EXPORT const char* heif_output_extensions[] = { "heif", "heic", "heics",
+OIIO_EXPORT const char* heif_output_extensions[] = { "heif",  "heic",
+                                                     "heics", "hif",
 #if LIBHEIF_HAVE_VERSION(1, 7, 0)
                                                      "avif",
 #endif
