@@ -1,5 +1,5 @@
-// Copyright 2008-present Contributors to the OpenImageIO project.
-// SPDX-License-Identifier: BSD-3-Clause
+// Copyright Contributors to the OpenImageIO project.
+// SPDX-License-Identifier: BSD-3-Clause and Apache-2.0
 // https://github.com/OpenImageIO/oiio
 
 
@@ -230,6 +230,10 @@
 #  define OIIO_MSVS_BEFORE_2015   (_MSC_VER <  1900)
 #  define OIIO_MSVS_AT_LEAST_2017 (_MSC_VER >= 1910)
 #  define OIIO_MSVS_BEFORE_2017   (_MSC_VER <  1910)
+#  define OIIO_MSVS_AT_LEAST_2019 (_MSC_VER >= 1920)
+#  define OIIO_MSVS_BEFORE_2019   (_MSC_VER <  1920)
+#  define OIIO_MSVS_AT_LEAST_2022 (_MSC_VER >= 1930)
+#  define OIIO_MSVS_BEFORE_2022   (_MSC_VER <  1930)
 #  if OIIO_MSVS_BEFORE_2017
 #    error "This version of OIIO is meant to work only with Visual Studio 2017 or later"
 #  endif
@@ -241,6 +245,10 @@
 #  define OIIO_MSVS_BEFORE_2015   0
 #  define OIIO_MSVS_AT_LEAST_2017 0
 #  define OIIO_MSVS_BEFORE_2017   0
+#  define OIIO_MSVS_AT_LEAST_2019 0
+#  define OIIO_MSVS_BEFORE_2019   0
+#  define OIIO_MSVS_AT_LEAST_2022 0
+#  define OIIO_MSVS_BEFORE_2022   0
 #endif
 
 
