@@ -1,5 +1,5 @@
-// Copyright 2008-present Contributors to the OpenImageIO project.
-// SPDX-License-Identifier: BSD-3-Clause
+// Copyright Contributors to the OpenImageIO project.
+// SPDX-License-Identifier: BSD-3-Clause and Apache-2.0
 // https://github.com/OpenImageIO/oiio
 
 #include <cmath>
@@ -877,7 +877,7 @@ ImageViewer::updateTitle()
         return;
     }
     std::string message;
-    message = Strutil::sprintf("%s - iv Image Viewer", img->name());
+    message = Strutil::fmt::format("{} - iv Image Viewer", img->name());
     setWindowTitle(QString::fromLocal8Bit(message.c_str()));
 }
 
@@ -2049,7 +2049,7 @@ ImageViewer::about()
         this, tr("About iv"),
         tr("<p><b>iv</b> is the image viewer for OpenImageIO.</p>"
            "<p>(c) Copyright Contributors to the OpenImageIO project.</p>"
-           "<p>See <a href='http://openimageio.org'>http://openimageio.org</a> for details.</p>"));
+           "<p>See <a href='https://openimageio.org'>https://openimageio.org</a> for details.</p>"));
 }
 
 
