@@ -1,5 +1,5 @@
-// Copyright 2008-present Contributors to the OpenImageIO project.
-// SPDX-License-Identifier: BSD-3-Clause
+// Copyright Contributors to the OpenImageIO project.
+// SPDX-License-Identifier: BSD-3-Clause and Apache-2.0
 // https://github.com/OpenImageIO/oiio
 
 #include <cassert>
@@ -408,7 +408,7 @@ JpgOutput::resmeta_to_density()
             // resolutions were not set
             if (aspect >= 1.0f) {
                 XRes = 72.0f;
-                YRes = XRes * aspect;
+                YRes = XRes / aspect;
             } else {
                 YRes = 72.0f;
                 XRes = YRes * aspect;
