@@ -1,5 +1,5 @@
-// Copyright 2008-present Contributors to the OpenImageIO project.
-// SPDX-License-Identifier: BSD-3-Clause
+// Copyright Contributors to the OpenImageIO project.
+// SPDX-License-Identifier: BSD-3-Clause and Apache-2.0
 // https://github.com/OpenImageIO/oiio
 
 #include <algorithm>
@@ -1510,8 +1510,15 @@ ImageBufAlgo::colorconvert(ImageBuf& dst, const ImageBuf& src, string_view from,
             if (colorconfig->error())
                 dst.errorfmt("{}", colorconfig->geterror());
             else
-                dst.errorfmt("Could not construct the color transform {} -> {}",
-                             from, to);
+#ifdef USE_OCIO
+                dst.errorfmt(
+                    "Could not construct the color transform {} -> {} (unknown error)",
+                    from, to);
+#else
+                dst.errorfmt(
+                    "Could not construct the color transform {} -> {} (no OpenColorIO support)",
+                    from, to);
+#endif
             return false;
         }
     }
@@ -1826,7 +1833,13 @@ ImageBufAlgo::ociolook(ImageBuf& dst, const ImageBuf& src, string_view looks,
             if (colorconfig->error())
                 dst.errorfmt("{}", colorconfig->geterror());
             else
-                dst.errorfmt("Could not construct the color transform");
+#ifdef USE_OCIO
+                dst.errorfmt(
+                    "Could not construct the color transform (unknown error)");
+#else
+                dst.errorfmt(
+                    "Could not construct the color transform (no OpenColorIO support)");
+#endif
             return false;
         }
     }
@@ -1886,7 +1899,13 @@ ImageBufAlgo::ociodisplay(ImageBuf& dst, const ImageBuf& src,
             if (colorconfig->error())
                 dst.errorfmt("{}", colorconfig->geterror());
             else
-                dst.errorfmt("Could not construct the color transform");
+#ifdef USE_OCIO
+                dst.errorfmt(
+                    "Could not construct the color transform (unknown error)");
+#else
+                dst.errorfmt(
+                    "Could not construct the color transform (no OpenColorIO support)");
+#endif
             return false;
         }
     }
@@ -1936,7 +1955,13 @@ ImageBufAlgo::ociofiletransform(ImageBuf& dst, const ImageBuf& src,
             if (colorconfig->error())
                 dst.errorfmt("{}", colorconfig->geterror());
             else
-                dst.errorfmt("Could not construct the color transform");
+#ifdef USE_OCIO
+                dst.errorfmt(
+                    "Could not construct the color transform (unknown error)");
+#else
+                dst.errorfmt(
+                    "Could not construct the color transform (no OpenColorIO support)");
+#endif
             return false;
         }
     }
