@@ -1,5 +1,5 @@
-// Copyright 2008-present Contributors to the OpenImageIO project.
-// SPDX-License-Identifier: BSD-3-Clause
+// Copyright Contributors to the OpenImageIO project.
+// SPDX-License-Identifier: BSD-3-Clause and Apache-2.0
 // https://github.com/OpenImageIO/oiio
 
 #include <cstdio>
@@ -36,7 +36,7 @@ threads_default()
 
 // Global private data
 namespace pvt {
-mutex imageio_mutex;
+std::recursive_mutex imageio_mutex;
 atomic_int oiio_threads(threads_default());
 atomic_int oiio_exr_threads(threads_default());
 atomic_int oiio_read_chunk(256);
@@ -231,12 +231,12 @@ pvt::append_error(string_view message)
     // a single newline.
     if (error_msg.size() && error_msg.back() != '\n')
         error_msg += '\n';
-    error_msg += message;
+    error_msg += std::string(message);
 
     // Remove a single trailing newline
     if (message.size() && message.back() == '\n')
         message.remove_suffix(1);
-    error_msg = message;
+    error_msg = std::string(message);
 }
 
 
@@ -502,6 +502,10 @@ getattribute(string_view name, TypeDesc type, void* val)
                                             (v >> 16) & 0xff, (v >> 8) & 0xff);
         return true;
     }
+    if (name == "opencv_version" && type == TypeInt) {
+        *(int*)val = OIIO::pvt::opencv_version;
+        return true;
+    }
     return false;
 }
 
