@@ -1,5 +1,5 @@
-// Copyright 2008-present Contributors to the OpenImageIO project.
-// SPDX-License-Identifier: BSD-3-Clause
+// Copyright Contributors to the OpenImageIO project.
+// SPDX-License-Identifier: BSD-3-Clause and Apache-2.0
 // https://github.com/OpenImageIO/oiio
 
 #include <cmath>
@@ -576,24 +576,10 @@ fix_latl_edges(ImageBuf& buf)
 
 
 
-static std::string
-formatres(const ImageSpec& spec, bool extended = false)
+inline std::string
+formatres(const ImageSpec& spec)
 {
-    std::string s;
-    s = Strutil::sprintf("%dx%d", spec.width, spec.height);
-    if (extended) {
-        if (spec.x || spec.y)
-            s += Strutil::sprintf("%+d%+d", spec.x, spec.y);
-        if (spec.width != spec.full_width || spec.height != spec.full_height
-            || spec.x != spec.full_x || spec.y != spec.full_y) {
-            s += " (full/display window is ";
-            s += Strutil::sprintf("%dx%d", spec.full_width, spec.full_height);
-            if (spec.full_x || spec.full_y)
-                s += Strutil::sprintf("%+d%+d", spec.full_x, spec.full_y);
-            s += ")";
-        }
-    }
-    return s;
+    return Strutil::fmt::format("{}x{}", spec.width, spec.height);
 }
 
 
@@ -629,6 +615,7 @@ write_mipmap(ImageBufAlgo::MakeTextureMode mode, std::shared_ptr<ImageBuf>& img,
              double& stat_miptime, size_t& peak_mem)
 {
     using OIIO::pvt::errorfmt;
+    using OIIO::Strutil::sync::print;  // Be sure to use synchronized one
     bool envlatlmode       = (mode == ImageBufAlgo::MakeTxEnvLatl);
     bool orig_was_overscan = (img->spec().x || img->spec().y || img->spec().z
                               || img->spec().full_x || img->spec().full_y
@@ -705,9 +692,10 @@ write_mipmap(ImageBufAlgo::MakeTextureMode mode, std::shared_ptr<ImageBuf>& img,
 
     // Write out the image
     if (verbose) {
-        outstream << "  Writing file: " << outputfilename << std::endl;
-        outstream << "  Filter \"" << filtername << "\"\n";
-        outstream << "  Top level is " << formatres(outspec) << std::endl;
+        print(outstream, "  Writing file: {}\n", outputfilename);
+        print(outstream, "  Filter \"{}\"\n", filtername);
+        print(outstream, "  Top level is {}x{}\n", outspec.width,
+              outspec.height);
     }
 
     if (clamp_half) {
@@ -723,11 +711,18 @@ write_mipmap(ImageBufAlgo::MakeTextureMode mode, std::shared_ptr<ImageBuf>& img,
         return false;
     }
 
-    stat_writetime += writetimer();
+    double wtime = writetimer();
+    stat_writetime += wtime;
+    if (verbose) {
+        size_t mem = Sysutil::memory_used(true);
+        peak_mem   = std::max(peak_mem, mem);
+        print(outstream, "    {:-15s} ({})  write {}\n", formatres(outspec),
+              Strutil::memformat(mem), Strutil::timeintervalformat(wtime, 2));
+    }
 
     if (mipmap) {  // Mipmap levels:
         if (verbose)
-            outstream << "  Mipmapping...\n" << std::flush;
+            print(outstream, "  Mipmapping...\n");
         std::vector<std::string> mipimages;
         std::string mipimages_unsplit = configspec.get_string_attribute(
             "maketx:mipimages");
@@ -807,16 +802,16 @@ write_mipmap(ImageBufAlgo::MakeTextureMode mode, std::shared_ptr<ImageBuf>& img,
                         return false;
                     }
                     if (verbose) {
-                        outstream << "  Downsampling filter \""
-                                  << filter->name()
-                                  << "\" width = " << filter->width();
-                        if (sharpen > 0.0f) {
-                            outstream << ", sharpening " << sharpen << " with "
-                                      << sharpenfilt << " unsharp mask "
-                                      << (sharpen_first ? "before" : "after")
-                                      << " the resize";
-                        }
-                        outstream << "\n";
+                        print(outstream,
+                              "  Downsampling filter \"{}\" width = {}",
+                              filter->name(), filter->width());
+                        if (sharpen > 0.0f)
+                            print(
+                                outstream,
+                                ", sharpening {} with {} unsharp mask {} the resize",
+                                sharpen, sharpenfilt,
+                                (sharpen_first ? "before" : "after"));
+                        print(outstream, "\n");
                     }
                     if (do_highlight_compensation)
                         ImageBufAlgo::rangecompress(*img, *img);
@@ -851,7 +846,8 @@ write_mipmap(ImageBufAlgo::MakeTextureMode mode, std::shared_ptr<ImageBuf>& img,
             if (clamp_half)
                 ImageBufAlgo::clamp(*small, *small, -HALF_MAX, HALF_MAX, true);
 
-            stat_miptime += miptimer();
+            double this_miptime = miptimer();
+            stat_miptime += this_miptime;
             outspec = smallspec;
             outspec.set_format(outputdatatype);
             if (envlatlmode && src_samples_border)
@@ -876,22 +872,23 @@ write_mipmap(ImageBufAlgo::MakeTextureMode mode, std::shared_ptr<ImageBuf>& img,
                 out->close();
                 return false;
             }
-            stat_writetime += writetimer();
+            double wtime = writetimer();
+            stat_writetime += wtime;
             if (verbose) {
                 size_t mem = Sysutil::memory_used(true);
                 peak_mem   = std::max(peak_mem, mem);
-                outstream << Strutil::sprintf("    %-15s (%s)",
-                                              formatres(smallspec),
-                                              Strutil::memformat(mem))
-                          << std::endl;
+                print(outstream, "    {:-15s} ({})  downres {} write {}\n",
+                      formatres(smallspec), Strutil::memformat(mem),
+                      Strutil::timeintervalformat(this_miptime, 2),
+                      Strutil::timeintervalformat(wtime, 2));
             }
             std::swap(img, small);
         }
     }
 
     if (verbose)
-        outstream << "  Wrote file: " << outputfilename << "  ("
-                  << Strutil::memformat(Sysutil::memory_used(true)) << ")\n";
+        print(outstream, "  Wrote file: {}  ({})\n", outputfilename,
+              Strutil::memformat(Sysutil::memory_used(true)));
     writetimer.reset();
     writetimer.start();
     if (!out->close()) {
@@ -961,7 +958,7 @@ stripdir_cmd_line(string_view cmdline)
         // Add the maybe-stripped string to the output, surrounding by
         // double quotes if it contains any spaces.
         if (stripped.find(' ') != std::string::npos)
-            out += Strutil::sprintf("\"%s\"", stripped);
+            out += Strutil::fmt::format("\"{}\"", stripped);
         else
             out += stripped;
         firstarg  = false;
@@ -978,6 +975,7 @@ make_texture_impl(ImageBufAlgo::MakeTextureMode mode, const ImageBuf* input,
                   const ImageSpec& _configspec, std::ostream* outstream_ptr)
 {
     using OIIO::pvt::errorfmt;
+    using OIIO::Strutil::sync::print;  // Be sure to use synchronized one
     OIIO_ASSERT(mode >= 0 && mode < ImageBufAlgo::_MakeTxLast);
     double stat_readtime         = 0;
     double stat_writetime        = 0;
@@ -987,15 +985,14 @@ make_texture_impl(ImageBufAlgo::MakeTextureMode mode, const ImageBuf* input,
     size_t peak_mem              = 0;
     Timer alltime;
 
-#define STATUS(task, timer)                                                  \
-    {                                                                        \
-        size_t mem = Sysutil::memory_used(true);                             \
-        peak_mem   = std::max(peak_mem, mem);                                \
-        if (verbose)                                                         \
-            outstream << Strutil::sprintf("  %-25s %s   (%s)\n", task,       \
-                                          Strutil::timeintervalformat(timer, \
-                                                                      2),    \
-                                          Strutil::memformat(mem));          \
+#define STATUS(task, timer)                                 \
+    {                                                       \
+        size_t mem = Sysutil::memory_used(true);            \
+        peak_mem   = std::max(peak_mem, mem);               \
+        if (verbose)                                        \
+            print(outstream, "  {:-25s} {}   ({})\n", task, \
+                  Strutil::timeintervalformat(timer, 2),    \
+                  Strutil::memformat(mem));                 \
     }
 
     ImageSpec configspec = _configspec;
@@ -1153,7 +1150,7 @@ make_texture_impl(ImageBufAlgo::MakeTextureMode mode, const ImageBuf* input,
         }
     }
     stat_readtime += alltime.lap();
-    STATUS(Strutil::sprintf("read \"%s\"", src->name()), stat_readtime);
+    STATUS(Strutil::fmt::format("read \"{}\"", src->name()), stat_readtime);
 
     if (mode == ImageBufAlgo::MakeTxEnvLatlFromLightProbe) {
         ImageSpec newspec = src->spec();
@@ -1174,7 +1171,8 @@ make_texture_impl(ImageBufAlgo::MakeTextureMode mode, const ImageBuf* input,
         src  = latlong;
     }
 
-    if (mode == ImageBufAlgo::MakeTxBumpWithSlopes) {
+    const bool is_bumpslopes = (mode == ImageBufAlgo::MakeTxBumpWithSlopes);
+    if (is_bumpslopes) {
         ImageSpec newspec  = src->spec();
         newspec.tile_width = newspec.tile_height = 0;
         newspec.set_format(TypeDesc::FLOAT);
@@ -1225,8 +1223,9 @@ make_texture_impl(ImageBufAlgo::MakeTextureMode mode, const ImageBuf* input,
         const float c_sigma_inv = fast_erf(1.0f / (2.0f * M_SQRT2 * cdf_sigma));
 
         // If there are channels other than R,G,B,A, we probably shouldn't do
-        // anything to them.
-        const int channels = std::min(4, src->spec().nchannels);
+        // anything to them, unless they are bumpslopes channels.
+        const int channels = is_bumpslopes ? 6
+                                           : std::min(4, src->spec().nchannels);
 
         std::vector<float> invCDF(bins);
         std::vector<float> CDF(bins);
@@ -1284,11 +1283,13 @@ make_texture_impl(ImageBufAlgo::MakeTextureMode mode, const ImageBuf* input,
     bool constant_color_detect = configspec.get_int_attribute(
         "maketx:constant_color_detect");
     bool opaque_detect = configspec.get_int_attribute("maketx:opaque_detect");
+    bool monochrome_detect = configspec.get_int_attribute(
+        "maketx:monochrome_detect");
     bool compute_average_color
         = configspec.get_int_attribute("maketx:compute_average", 1);
     ImageBufAlgo::PixelStats pixel_stats;
     bool compute_stats = (constant_color_detect || opaque_detect
-                          || compute_average_color);
+                          || compute_average_color || monochrome_detect);
     if (compute_stats) {
         pixel_stats = ImageBufAlgo::computePixelStats(*src);
     }
@@ -1346,14 +1347,21 @@ make_texture_impl(ImageBufAlgo::MakeTextureMode mode, const ImageBuf* input,
         std::swap(src, newsrc);  // N.B. the old src will delete
     }
 
-    // If requested - and we're a monochrome image - drop the extra channels
-    if (configspec.get_int_attribute("maketx:monochrome_detect")
-        && nchannels <= 0 && src->nchannels() == 3
-        && src->spec().alpha_channel < 0 &&  // RGB only
-        ImageBufAlgo::isMonochrome(*src)) {
+    // If requested - and we're a monochrome image - drop the extra channels.
+    // In addition to only doing this for RGB images (3 channels, no alpha),
+    // we also check the stat averages are the same for all three channels (if
+    // the channel averages are not identical, they surely cannot be the same
+    // for all pixels, so there is no point wasting the time of the call to
+    // isMonochrome().
+    if (monochrome_detect && nchannels <= 0 && src->nchannels() == 3
+        && src->spec().alpha_channel < 0
+        && pixel_stats.avg[0] == pixel_stats.avg[1]
+        && pixel_stats.avg[0] == pixel_stats.avg[2]
+        && ImageBufAlgo::isMonochrome(*src)) {
         if (verbose)
-            outstream
-                << "  Monochrome image detected. Converting to single channel texture.\n";
+            print(
+                outstream,
+                "  Monochrome image detected. Converting to single channel texture.\n");
         std::shared_ptr<ImageBuf> newsrc(new ImageBuf(src->spec()));
         ImageBufAlgo::channels(*newsrc, *src, 1, NULL, NULL, NULL, true);
         newsrc->specmod().default_channel_names();
@@ -1365,7 +1373,7 @@ make_texture_impl(ImageBufAlgo::MakeTextureMode mode, const ImageBuf* input,
     if ((nchannels > 0) && (nchannels != src->nchannels())) {
         if (verbose)
             outstream << "  Overriding number of channels to " << nchannels
-                      << "\n";
+                      << std::endl;
         std::shared_ptr<ImageBuf> newsrc(new ImageBuf(src->spec()));
         ImageBufAlgo::channels(*newsrc, *src, nchannels, NULL, NULL, NULL,
                                true);
@@ -1661,14 +1669,6 @@ make_texture_impl(ImageBufAlgo::MakeTextureMode mode, const ImageBuf* input,
         STATUS("color convert", stat_colorconverttime);
     }
 
-    // Force float for the sake of the ImageBuf math.
-    // Also force float if we do not allow for the pixel shift,
-    // since resize_block_ requires floating point buffers.
-    const int allow_shift = configspec.get_int_attribute(
-        "maketx:allow_pixel_shift");
-    if (configspec.get_int_attribute("maketx:forcefloat", 1) || !allow_shift)
-        dstspec.set_format(TypeDesc::FLOAT);
-
     // Handle resize to power of two, if called for
     if (configspec.get_int_attribute("maketx:resize") && !shadowmode) {
         dstspec.width       = ceil2(dstspec.width);
@@ -1690,6 +1690,15 @@ make_texture_impl(ImageBufAlgo::MakeTextureMode mode, const ImageBuf* input,
             || Strutil::iends_with(outputfilename, ".exr")))
         do_resize = true;
 
+    // Force float for the sake of the ImageBuf math.
+    // Also force float if we do not allow for the pixel shift,
+    // since resize_block_ requires floating point buffers.
+    const int allow_shift = configspec.get_int_attribute(
+        "maketx:allow_pixel_shift");
+    if (configspec.get_int_attribute("maketx:forcefloat", 1)
+        || (do_resize && !allow_shift))
+        dstspec.set_format(TypeDesc::FLOAT);
+
     if (orig_was_overscan && out && !out->supports("displaywindow")) {
         errorfmt(
             "Format \"{}\" does not support separate display "
@@ -1711,13 +1720,16 @@ make_texture_impl(ImageBufAlgo::MakeTextureMode mode, const ImageBuf* input,
         toplevel = src;
     } else if (!do_resize) {
         // Need format conversion, but no resize -- just copy the pixels
+        if (verbose)
+            print(outstream, "  Copying for format conversion from {} to {}\n",
+                  src->spec().format, dstspec.format);
         toplevel.reset(new ImageBuf(dstspec));
         toplevel->copy_pixels(*src);
     } else {
         // Resize
         if (verbose)
-            outstream << "  Resizing image to " << dstspec.width << " x "
-                      << dstspec.height << std::endl;
+            print(outstream, "  Resizing image to {} x {}\n", dstspec.width,
+                  dstspec.height);
         string_view resize_filter(filtername);
         if (Strutil::istarts_with(resize_filter, "unsharp-"))
             resize_filter = "lanczos3";
@@ -1816,8 +1828,8 @@ make_texture_impl(ImageBufAlgo::MakeTextureMode mode, const ImageBuf* input,
         if (out->supports("arbitrary_metadata")) {
             dstspec.attribute("oiio:ConstantColor", colstr);
         } else {
-            desc += Strutil::sprintf("%soiio:ConstantColor=%s",
-                                     desc.length() ? " " : "", colstr);
+            desc += Strutil::fmt::format("{}oiio:ConstantColor={}",
+                                         desc.length() ? " " : "", colstr);
             updatedDesc = true;
         }
         if (verbose)
@@ -1832,8 +1844,8 @@ make_texture_impl(ImageBufAlgo::MakeTextureMode mode, const ImageBuf* input,
         } else {
             // if arbitrary metadata is not supported, cram it into the
             // ImageDescription.
-            desc += Strutil::sprintf("%soiio:AverageColor=%s",
-                                     desc.length() ? " " : "", avgstr);
+            desc += Strutil::fmt::format("{}oiio:AverageColor={}",
+                                         desc.length() ? " " : "", avgstr);
             updatedDesc = true;
         }
         if (verbose)
@@ -1894,28 +1906,22 @@ make_texture_impl(ImageBufAlgo::MakeTextureMode mode, const ImageBuf* input,
     if (verbose || configspec.get_int_attribute("maketx:runstats")
         || configspec.get_int_attribute("maketx:stats")) {
         double all = alltime();
-        Strutil::fprintf(outstream, "maketx run time (seconds): %5.2f\n", all);
-        Strutil::fprintf(outstream, "  file read:       %5.2f\n",
-                         stat_readtime);
-        Strutil::fprintf(outstream, "  file write:      %5.2f\n",
-                         stat_writetime);
-        Strutil::fprintf(outstream, "  initial resize:  %5.2f\n",
-                         stat_resizetime);
-        Strutil::fprintf(outstream, "  hash:            %5.2f\n",
-                         stat_hashtime);
-        Strutil::fprintf(outstream, "  pixelstats:      %5.2f\n",
-                         stat_pixelstatstime);
-        Strutil::fprintf(outstream, "  mip computation: %5.2f\n", stat_miptime);
-        Strutil::fprintf(outstream, "  color convert:   %5.2f\n",
-                         stat_colorconverttime);
-        Strutil::fprintf(
+        print(outstream, "maketx run time (seconds): {:5.2f}\n", all);
+        print(outstream, "  file read:       {:5.2f}\n", stat_readtime);
+        print(outstream, "  file write:      {:5.2f}\n", stat_writetime);
+        print(outstream, "  initial resize:  {:5.2f}\n", stat_resizetime);
+        print(outstream, "  hash:            {:5.2f}\n", stat_hashtime);
+        print(outstream, "  pixelstats:      {:5.2f}\n", stat_pixelstatstime);
+        print(outstream, "  mip computation: {:5.2f}\n", stat_miptime);
+        print(outstream, "  color convert:   {:5.2f}\n", stat_colorconverttime);
+        print(
             outstream,
-            "  unaccounted:     %5.2f  (%5.2f %5.2f %5.2f %5.2f %5.2f)\n",
+            "  unaccounted:     {:5.2f}  ({:5.2f} {:5.2f} {:5.2f} {:5.2f} {:5.2f})\n",
             all - stat_readtime - stat_writetime - stat_resizetime
                 - stat_hashtime - stat_miptime,
             misc_time_1, misc_time_2, misc_time_3, misc_time_4, misc_time_5);
-        Strutil::fprintf(outstream, "maketx peak memory used: %s\n",
-                         Strutil::memformat(peak_mem));
+        print(outstream, "maketx peak memory used: {}\n",
+              Strutil::memformat(peak_mem));
     }
 
 #undef STATUS
