@@ -1,5 +1,5 @@
-// Copyright 2008-present Contributors to the OpenImageIO project.
-// SPDX-License-Identifier: BSD-3-Clause
+// Copyright Contributors to the OpenImageIO project.
+// SPDX-License-Identifier: BSD-3-Clause and Apache-2.0
 // https://github.com/OpenImageIO/oiio
 
 #include <algorithm>
@@ -31,6 +31,7 @@
 #    include <sys/stat.h>
 #    include <sys/types.h>
 #    include <unistd.h>
+#    include <utime.h>
 #endif
 
 #if defined(USE_STD_FILESYSTEM)
@@ -708,12 +709,10 @@ Filesystem::last_write_time(string_view path, std::time_t time) noexcept
     times.modtime = time;
     _wutime(u8path(path).c_str(), &times);
 #else
-    struct timespec times[2];
-    times[0].tv_sec  = 0;
-    times[0].tv_nsec = UTIME_OMIT;
-    times[1].tv_sec  = time;
-    times[1].tv_nsec = 0;
-    utimensat((int)AT_FDCWD, u8path(path).c_str(), times, AT_SYMLINK_NOFOLLOW);
+    struct utimbuf times;
+    times.actime  = time;
+    times.modtime = time;
+    utime(u8path(path).c_str(), &times);
 #endif
 }
 
