@@ -1,5 +1,5 @@
-// Copyright 2008-present Contributors to the OpenImageIO project.
-// SPDX-License-Identifier: BSD-3-Clause
+// Copyright Contributors to the OpenImageIO project.
+// SPDX-License-Identifier: BSD-3-Clause and Apache-2.0
 // https://github.com/OpenImageIO/oiio
 
 #include <string>
@@ -181,7 +181,7 @@ private:
 
     char* pool_alloc(size_t len)
     {
-        // round up to nearest multiple of pointer size to guarentee proper alignment of TableRep objects
+        // round up to nearest multiple of pointer size to guarantee proper alignment of TableRep objects
         len = (len + alignof(ustring::TableRep) - 1)
               & ~(alignof(ustring::TableRep) - 1);
 
@@ -259,7 +259,7 @@ struct UstringTable {
 
 private:
     enum {
-        // NOTE: this guarentees NUM_BINS is a power of 2
+        // NOTE: this guarantees NUM_BINS is a power of 2
         BIN_SHIFT = 12,
         NUM_BINS  = 1 << BIN_SHIFT,
         TOP_SHIFT = 8 * sizeof(size_t) - BIN_SHIFT
@@ -388,7 +388,7 @@ ustring::TableRep::TableRep(string_view strref, size_t hash)
     // It turns out that the first field of a gcc std::string is a pointer
     // to the characters within the basic_string::_Rep.  We merely redirect
     // that pointer, though for std::string to function properly, the chars
-    // must be preceeded immediately in memory by the rest of
+    // must be preceded immediately in memory by the rest of
     // basic_string::_Rep, consisting of length, capacity and refcount
     // fields.  And we have designed our TableRep to do just that!  So now
     // we redirect the std::string's pointer to our own characters and its
@@ -417,7 +417,7 @@ ustring::TableRep::TableRep(string_view strref, size_t hash)
     // allocations to be had for short strings, so we just let those behave
     // as normal.  But if it's going to make a long string (we can tell from
     // the length), we construct it ourselves, forcing the pointer to point
-    // to the charcters in the TableRep we allocated.
+    // to the characters in the TableRep we allocated.
     if (length >= libcpp_string__min_cap /* it'll be a "long string" */) {
         ((libcpp_string__long*)&str)->__cap_ = libcpp_string__long_mask
                                                | (length + 1);
@@ -453,7 +453,7 @@ const char*
 ustring::make_unique(string_view strref)
 {
     UstringTable& table(ustring_table());
-    // Eliminate NULLs
+    // Eliminate nullptr-referred string views
     if (!strref.data())
         strref = string_view("", 0);
 
@@ -468,6 +468,18 @@ ustring::make_unique(string_view strref)
     const char* result = table.lookup(strref, hash);
     if (result)
         return result;
+    auto nul = strref.find('\0');
+    if (nul != string_view::npos) {
+        // Strutil::print("ustring::make_unique: string contains nulls @{}/{}: \"{}\"\n",
+        //                strref.find('\0'), strref.size(), strref);
+        // OIIO_ASSERT(strref.find('\0') == string_view::npos &&
+        //             "ustring::make_unique() does not support embedded nulls");
+        strref = strref.substr(0, nul);
+        hash   = Strutil::strhash(strref);
+        result = table.lookup(strref, hash);
+        if (result)
+            return result;
+    }
     // Strutil::print("ADDED ustring \"{}\" {:08x}\n", strref, hash);
     return table.insert(strref, hash);
 
@@ -480,6 +492,21 @@ ustring::make_unique(string_view strref)
     if (result)
         return result;
 
+    // ustring doesn't allow strings with embedded nul characters. Before we
+    // go any further, trim beyond any nul and rehash.
+    auto nul = strref.find('\0');
+    if (nul != string_view::npos) {
+        // Strutil::print("ustring::make_unique: string contains nulls @{}/{}: \"{}\"\n",
+        //                strref.find('\0'), strref.size(), strref);
+        // OIIO_ASSERT(strref.find('\0') == string_view::npos &&
+        //             "ustring::make_unique() does not support embedded nulls");
+        strref = strref.substr(0, nul);
+        hash   = Strutil::strhash(strref);
+        result = table.lookup(strref, hash);
+        if (result)
+            return result;
+    }
+
     // We did not find it. There are two possibilities: (1) the string is in
     // the table but has a different hash because it collided; or (2) the
     // string is not yet in the table.
@@ -516,7 +543,7 @@ ustring::make_unique(string_view strref)
             result = rev->second;
             break;
         }
-        // Rehash, but keep the bin bits identical so we alwasy rehash into
+        // Rehash, but keep the bin bits identical so we always rehash into
         // the same (locked) bin.
         hash = (hash & binmask)
                | (farmhash::Fingerprint(hash) & rm.nobin_mask());
@@ -573,15 +600,15 @@ ustring::concat(string_view s, string_view t)
 std::string
 ustring::getstats(bool verbose)
 {
-    UstringTable& table(ustring_table());
     std::ostringstream out;
     out.imbue(std::locale::classic());  // Force "C" locale with '.' decimal
-    size_t n_e = table.get_num_entries();
-    size_t mem = table.get_memory_usage();
+    size_t n_e = total_ustrings();
+    size_t mem = memory();
     if (verbose) {
         out << "ustring statistics:\n";
 #ifdef USTRING_TRACK_NUM_LOOKUPS
-        out << "  ustring requests: " << table.get_num_lookups() << "\n";
+        out << "  ustring requests: " << ustring_table().get_num_lookups()
+            << "\n";
 #endif
         out << "  unique strings: " << n_e << "\n";
         out << "  ustring memory: " << Strutil::memformat(mem) << "\n";
@@ -596,7 +623,7 @@ ustring::getstats(bool verbose)
 #endif
     } else {
 #ifdef USTRING_TRACK_NUM_LOOKUPS
-        out << "requests: " << table.get_num_lookups() << ", ";
+        out << "requests: " << ustring_table().get_num_lookups() << ", ";
 #endif
         out << "unique " << n_e << ", " << Strutil::memformat(mem);
     }
