@@ -1,5 +1,5 @@
-// Copyright 2008-present Contributors to the OpenImageIO project.
-// SPDX-License-Identifier: BSD-3-Clause
+// Copyright Contributors to the OpenImageIO project.
+// SPDX-License-Identifier: Apache-2.0
 // https://github.com/OpenImageIO/oiio
 
 
@@ -15,6 +15,9 @@
 #    include <io.h>
 #endif
 
+#include <OpenImageIO/detail/fmt.h>
+#include <OpenImageIO/half.h>
+
 #include "oiiotool.h"
 
 #include <OpenImageIO/Imath.h>
@@ -32,6 +35,17 @@ using namespace OiioTool;
 using namespace ImageBufAlgo;
 
 
+// clang-format off
+#if defined(FMT_VERSION) && !defined(OIIO_HALF_FORMATTER)
+#if FMT_VERSION >= 100000
+#define OIIO_HALF_FORMATTER
+FMT_BEGIN_NAMESPACE
+template<> struct formatter<half> : ostream_formatter { };
+FMT_END_NAMESPACE
+#endif
+#endif
+// clang-format on
+
 
 template<typename T>
 static void
