@@ -1,5 +1,5 @@
 // Copyright 2021-present Contributors to the OpenImageIO project.
-// SPDX-License-Identifier: BSD-3-Clause
+// SPDX-License-Identifier: BSD-3-Clause and Apache-2.0
 // https://github.com/OpenImageIO/oiio
 
 #include <cerrno>
@@ -956,8 +956,8 @@ OpenEXRCoreInput::PartInfo::query_channels(OpenEXRCoreInput* in,
             spec.z_channel = c;
         if (cnh[c].xSampling != 1 || cnh[c].ySampling != 1) {
             ok = false;
-            in->errorf(
-                "Subsampled channels are not supported (channel \"%s\" has sampling %d,%d).",
+            in->errorfmt(
+                "Subsampled channels are not supported (channel \"{}\" has sampling {},{}).",
                 cnh[c].fullname, cnh[c].xSampling, cnh[c].ySampling);
             // FIXME: Some day, we should handle channel subsampling.
         }
@@ -1203,7 +1203,7 @@ OpenEXRCoreInput::read_native_scanlines(int subimage, int miplevel, int ybegin,
             int y             = std::max(int(yb), ybegin);
             uint8_t* linedata = static_cast<uint8_t*>(data)
                                 + scanlinebytes * (y - ybegin);
-            std::unique_ptr<uint8_t> fullchunk;
+            std::unique_ptr<uint8_t[]> fullchunk;
             int nlines = scansperchunk;
             exr_chunk_info_t cinfo;
             exr_decode_pipeline_t decoder = EXR_DECODE_PIPELINE_INITIALIZER;
