@@ -1,5 +1,5 @@
-// Copyright 2008-present Contributors to the OpenImageIO project.
-// SPDX-License-Identifier: BSD-3-Clause
+// Copyright Contributors to the OpenImageIO project.
+// SPDX-License-Identifier: BSD-3-Clause and Apache-2.0
 // https://github.com/OpenImageIO/oiio
 
 #include <cerrno>
@@ -722,9 +722,10 @@ OpenEXROutput::spec_to_header(ImageSpec& spec, int subimage,
     // on deep files (but allow "none" as well)
     if (spec.deep && comp != "none")
         comp = "zips";
-    // For single channel images, dwaa/b compression only seems to work reliably
-    // when size > 16 and size is a power of two
-    if (spec.nchannels == 1 && Strutil::istarts_with(comp, "dwa")
+    // For single channel tiled images, dwaa/b compression only seems to work
+    // reliably when tile size > 16 and size is a power of two.
+    if (spec.nchannels == 1 && spec.tile_width > 0
+        && Strutil::istarts_with(comp, "dwa")
         && ((spec.tile_width < 16 && spec.tile_height < 16)
             || !ispow2(spec.tile_width) || !ispow2(spec.tile_height))) {
         comp = "zip";
@@ -1019,8 +1020,8 @@ OpenEXROutput::put_parameter(const std::string& name, TypeDesc type,
     // Now if we still don't match a specific type OpenEXR is looking for,
     // skip it.
     if (exrtype != TypeDesc() && !exrtype.equivalent(type)) {
-        OIIO::debugf(
-            "OpenEXR output metadata \"%s\" type mismatch: expected %s, got %s\n",
+        OIIO::debugfmt(
+            "OpenEXR output metadata \"{}\" type mismatch: expected {}, got {}\n",
             name, exrtype, type);
         return false;
     }
@@ -1288,12 +1289,12 @@ OpenEXROutput::put_parameter(const std::string& name, TypeDesc type,
 #endif
         }
     } catch (const std::exception& e) {
-        OIIO::debugf("Caught OpenEXR exception: %s\n", e.what());
+        OIIO::debugfmt("Caught OpenEXR exception: {}\n", e.what());
     } catch (...) {  // catch-all for edge cases or compiler bugs
         OIIO::debug("Caught unknown OpenEXR exception\n");
     }
 
-    OIIO::debugf("Don't know what to do with %s %s\n", type, xname);
+    OIIO::debugfmt("Don't know what to do with {} {}\n", type, xname);
     return false;
 }
 
