@@ -1,5 +1,5 @@
-// Copyright 2008-present Contributors to the OpenImageIO project.
-// SPDX-License-Identifier: BSD-3-Clause
+// Copyright Contributors to the OpenImageIO project.
+// SPDX-License-Identifier: BSD-3-Clause and Apache-2.0
 // https://github.com/OpenImageIO/oiio
 
 #include <algorithm>
@@ -433,8 +433,17 @@ RawInput::open_raw(bool unpack, const std::string& name,
         m_unpacked = true;
     }
 
-    //Store flip before it is potentially overwritten
-    int original_flip = m_processor->imgdata.sizes.flip;
+    // Store flip before it is potentially overwritten
+    // LibRaw's convention for flip values differs from Exif orientation tags
+    // so we need to convert it
+    int original_flip = 1;
+    int libraw_flip   = m_processor->imgdata.sizes.flip;
+    switch (libraw_flip) {
+    case 3: original_flip = 3; break;
+    case 5: original_flip = 8; break;
+    case 6: original_flip = 6; break;
+    default: break;
+    }
     m_processor->adjust_sizes_info_only();
 
     // Process image at half size if "raw:half_size" is not 0
@@ -445,8 +454,7 @@ RawInput::open_raw(bool unpack, const std::string& name,
     // Set file information
     m_spec = ImageSpec(m_processor->imgdata.sizes.iwidth / div,
                        m_processor->imgdata.sizes.iheight / div,
-                       3,  // LibRaw should only give us 3 channels
-                       TypeDesc::UINT16);
+                       m_processor->imgdata.idata.colors, TypeDesc::UINT16);
     // Move the exif attribs we already read into the spec we care about
     m_spec.extra_attribs.swap(exifspec.extra_attribs);
 
@@ -914,7 +922,7 @@ RawInput::open_raw(bool unpack, const std::string& name,
     add("Exif", "Pressure", common.exifPressure, false, 0.0f);
     add("Exif", "WaterDepth", common.exifWaterDepth, false, 0.0f);
     add("Exif", "Acceleration", common.exifAcceleration, false, 0.0f);
-    add("Exif", "CameraElevactionAngle", common.exifCameraElevationAngle, false,
+    add("Exif", "CameraElevationAngle", common.exifCameraElevationAngle, false,
         0.0f);
     // float real_ISO;
 #elif LIBRAW_VERSION >= LIBRAW_MAKE_VERSION(0, 19, 0)
@@ -931,7 +939,7 @@ RawInput::open_raw(bool unpack, const std::string& name,
     add("Exif", "Pressure", other.exifPressure, false, 0.0f);
     add("Exif", "WaterDepth", other.exifWaterDepth, false, 0.0f);
     add("Exif", "Acceleration", other.exifAcceleration, false, 0.0f);
-    add("Exif", "CameraElevactionAngle", other.exifCameraElevationAngle, false,
+    add("Exif", "CameraElevationAngle", other.exifCameraElevationAngle, false,
         0.0f);
     // float real_ISO;
 #endif
@@ -944,10 +952,10 @@ RawInput::open_raw(bool unpack, const std::string& name,
         m_spec.attribute("raw:Orientation", ori);
     m_spec.attribute("Orientation", 1);
 
-    // If user flip is set to 1, it means we ignore the flip
+    // If user flip is set to 0, it means we ignore the flip
     // Let's set the orientation exif flags to the original flip
     // value so that it is still displayed correctly
-    if (config.get_int_attribute("raw:user_flip", -1) == 1) {
+    if (config.get_int_attribute("raw:user_flip", -1) == 0) {
         m_spec.attribute("Orientation", original_flip);
     }
 
@@ -1492,9 +1500,8 @@ RawInput::process()
             errorf("LibRaw did not return expected image type");
             return false;
         }
-
-        if (m_image->colors != 3) {
-            errorf("LibRaw did not return 3 channel image");
+        if (m_image->colors != 1 && m_image->colors != 3) {
+            errorf("LibRaw did not return a 1 or 3 channel image");
             return false;
         }
     }
